/* 
 * ap-qsort.c 
 * - Implementation of MultiKey QuickSort of Bentry and Sedgewick.
 * - 08 DEC 98 created and highly tuned by Toru Kasai. 
 * - 21 SEP 99 by arim: fix bugs in main sort routines of Kasai, 
 *   which greatly improves its efficiency. (Treatment of a doc separator)
 * $Id: ap-qsort.c,v 1.1.1.1 2004/02/14 04:30:12 arim Exp $
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <limits.h>
#include "ap-qsort.h"

/******************************************************************
 * Sedgewick's ternary quick sort:
 * - "Fast algorithms for sorting and searching strings", 
 * - Bentry and Sedgewick, Proc. SODA (1995).
 * - A fast algorithm particularly quick for long strings. 
 ******************************************************************/

/******************************************************************
 *  A quick sort routine:
 * - implemented by Tohru Kasai. 
 ******************************************************************/

#ifndef min
#define min(a, b) ((a)<=(b) ? (a) : (b))
#endif

void vecswap2(int *sary, int *b, int n)
{   while (n-- > 0) {
        int t = *sary;
        *(sary++) = *b;
        *(b++) = t;
    }
}

#define swap2(a, b) { t = *(a); *(a) = *(b); *(b) = t; }

int *QSmed3func(int *a, int *b, int *c)
{   int va, vb, vc;
    if ((va=*a) == (vb=*b))
        return a;
    if ((vc=*c) == va || vc == vb)
        return c;       
    return va < vb ?
          (vb < vc ? b : (va < vc ? c : a ) )
        : (vb > vc ? b : (va < vc ? a : c ) );
}
#define QSmed3(a, b, c) QSmed3func(a, b, c)

void QSinssort(int *IAry, int n)
{   
    int *pi, *pj, t;
    for (pi = IAry+1; --n > 0; pi++)
        for (pj = pi; pj > IAry; pj--) {
            if (*(pj-1) < *pj)
                break;
	    if (*(pj-1) == *pj)
		break;
            swap2(pj, pj-1);
    }
}

void QSort(int *IA, int n)
{   /* - A quick sort routine for sorting the integer array of
     *  length n
     */
    int r, partval, d;
    int  *pa, *pb, *pc, *pd, *pl, *pm, *pn, t;
    if (n < 10) {
        QSinssort(IA, n);
        return;
    }
    pl = IA;
    pm = IA + (n/2);
    pn = IA + (n-1);
    if (n > 30) { // On big arrays, pseudomedian of 9
        d = (n/8);
        pl = QSmed3(pl, pl+d, pl+2*d);
        pm = QSmed3(pm-d, pm, pm+d);
        pn = QSmed3(pn-2*d, pn-d, pn);
    }
    pm = QSmed3(pl, pm, pn);
    swap2(IA, pm);
    partval = *IA;
    pa = pb = IA + 1;
    pc = pd = IA + n-1;
    for (;;) {
        while (pb <= pc && (r = *pb-partval) <= 0) {
            if (r == 0) { swap2(pa, pb); pa++; }
            pb++;
        }
        while (pb <= pc && (r = *pc-partval) >= 0) {
            if (r == 0) { swap2(pc, pd); pd--; }
            pc--;
        }
        if (pb > pc) break;
        swap2(pb, pc);
        pb++;
        pc--;
    }
    pn = IA + n;
    r = min(pa-IA, pb-pa);    vecswap2(IA,  pb-r, r);
    r = min(pd-pc, pn-pd-1); vecswap2(pb, pn-r, r);
    if ((r = pb-pa) > 1)
        QSort(IA, r);
    if ((r = pd-pc) > 1)
        QSort(IA + n-r, r);
}

/******************************************************************
 * End
 * 
 ******************************************************************/

