/******************************************************************
 * ep-approx-list.c
 * - ߎ}CjOASYD
 *   ɁCҏW{insert, delete, exchange}gD
 * - A frequent sequence miner for approximate string patterns
 *   where {insert, delete, exchange}-operations are allowed
 *
 * Modified on 25 FEB 2004 by arim
 * - ̐炷߂ -l IvViŒҏWjgݍ񂾁D
 *   l = 0 ̏ꍇ́Cp^[̓eLXg̕ɐ̂ŁC
 *   n^2̉ȂDʂɁC𐔂͍Xn^{l+2}łD
 * Modified on 14 FEB 2004 by arim
 * - R}hCƃIvVŁCp[^ǂݍ߂悤ɂD
 *   requires GNU getopt library (contained in gcc package).
 * Created on 14 FEB 2004 by arim@i.kyushu-u.ac.jp
 *   based on ep-approx-list-all2.c 
 *   ep-approx-list-all2.c: 
 *   Created on 4 FEB 2004 by {uchida,arim}@i.kyushu-u.ac.jp.
 *   based on ep-approx-list-all.c 
 * $Id: main.c,v 1.6 2004/02/25 20:31:59 arim Exp $
 ******************************************************************/

#include "ap-util.h"
#include "ap-carray.h"
#include "ap-iarray.h"
#include <assert.h>
#include <stdlib.h>
#include <time.h>
#include <getopt.h> /* for gnu-dependent getopt-long library */
#include "ep-mine-api.h"

/******************************************************************
 * Macros
 ******************************************************************/

/* For gnuopt
 */
#define EOFOPT (-1) /* End of option */

/* General variables
 */

int MAXINS = INTMAX;
int MAXDEL = INTMAX;
int MAXEX  = INTMAX;
int DELTA_WIN  = 0; //default
int ENABLE_PRINT_PATTERN  = TRUE;
int WINDOW_FREE = FALSE;

/* General variables*/
char filename[128];
int   minsup = 0; 	/* minimum support in */
float minsup_percent = 0; 	/* minimum support in percent */
int   maxscore = 0;                /* maximum edit distance */
int   minscore = INTMAX;           /* minimum edit distance */
int   k = 0;                /* Only for compatibility */

/******************************************************************
 * Global
 ******************************************************************/

//int GAMMA[20][20];

/* Usage: vO̎g */
char USAGE_STRING[] = 
"usage: main OPTIONS inputfile\n"
"OPTIONS: \n"
" -k maxscore    \t maximum integer score allowed (default 0)\n"
" -s minsup(num) \t minimum support in the number of strings (default 0)\n"
" -r minsup(ratio)) \t minimum support in percent (from 0 to 1)\n"
" -i | -d | -e   \t Using ins/del/ex oprs with score 1 (default false)\n" 
"(-I|-D|-E) maxscore\t Using the operation with score maxscore\n"
" -w             \t Using windowed epsode with width k(maxscore) (default false)\n" 
" -f             \t Using window-free epsode (default false)\n" 
" -l minscore    \t At least one string must contain the pattern in minscore (array only)\n"
" -p             \t printing patterns (default)\n"
" -n             \t not printing patterns\n"
" Note: you can use x/X instead of e/E for the exchange operation\n" 
; /* End of USAGE_STRING */


/* IvV̏w */
static char* OptionFormat = "hidexwI:D:E:X:W:s:r:k:l:pnf"; 

/******************************************************************
 * Main
 ******************************************************************/

void printAnOption(char *str, int optind, char *optarg) {
#ifdef DEBUG
  printf("#%s: optind=%d optarg=[%s]\n", str, optind, optarg);
#else
  ;
#endif
}


void readOptions(int argc, char *argv[]) {
  /* Option variables */
  int C; /* Option letter */
  extern int optind; 
  extern char *optarg;

  if (argc <= 1) { /* no arguments */
    printf(USAGE_STRING);
    exit(0);
  }

  while ((C = getopt(argc, argv, OptionFormat)) != EOFOPT) {
#ifdef DEBUG
    printf("#getopt: C=%d\n", C);
#endif
    switch (C) {
    case 's':
      /* -s minsup: the number of input sequences as minimum support */
      printAnOption("s", optind, optarg);
      minsup = atoi(optarg);
      break;
    case 'r':
      /* -r minsup: the percent as minimum support */
      printAnOption("r", optind, optarg);
      minsup = 0;
      minsup_percent = atof(optarg);
      break;
    case 'k':
      /* -k maxscore: the percent as minimum support */
      printAnOption("k", optind, optarg);
      maxscore = atoi(optarg);
      k = maxscore; /* Only for compatiblity */
      break;
      //Flag for edit operations
    case 'l':
      /* -l minscore: the percent as minimum support */
      printAnOption("l", optind, optarg);
      minscore = atoi(optarg);
      break;
      //Flag for edit operations
    case 'i':
      printAnOption("i", optind, optarg);
      //USEINS = TRUE;
      MAXINS = 1;
      break;
    case 'd':
      printAnOption("d", optind, optarg);
      //USEDEL = TRUE;
      MAXDEL = 1;
      break;
    case 'e':
    case 'x':
      printAnOption("e", optind, optarg);
      //USEEX = TRUE;
      MAXEX  = 1;
      break;
      //Set non-standard edit score
    case 'w':
      printAnOption("W", optind, optarg);
      //USEINS = TRUE;
      MAXINS = 1;
      MAXDEL = INTMAX;
      MAXEX  = INTMAX;
      DELTA_WIN = 1;
      break;
    case 'f':
      printAnOption("F", optind, optarg);
      WINDOW_FREE = TRUE;
      maxscore = INTMAX;
      k = maxscore; /* Only for compatiblity */
      break;
    case 'I':
      printAnOption("I", optind, optarg);
      //USEINS = TRUE;
      MAXINS = atoi(optarg);
      break;
    case 'D':
      printAnOption("D", optind, optarg);
      //USEDEL = TRUE;
      MAXDEL = atoi(optarg);
      break;
    case 'E':
    case 'X':
      printAnOption("E", optind, optarg);
      //USEEX = TRUE;
      MAXEX = atoi(optarg);
      break;
    case 'p':
      printAnOption("p", optind, optarg);
      ENABLE_PRINT_PATTERN = TRUE;
      break;
    case 'n':
      printAnOption("n", optind, optarg);
      ENABLE_PRINT_PATTERN = FALSE;
      break;
    case 'h':
      printAnOption("h", optind, optarg);
      printf(USAGE_STRING);
      exit(0);
    default:
      printAnOption("*", optind, optarg);
      printf("#getopt returned char: %d\n", C);
      exit(0);
    }
  }
}

void readArgs(int argc, char *argv[]) {
  int  _argc   = 0;
  char **_argv = NULL;
  int i;
  char *str;
  _argc = argc - optind;
  _argv = argv + optind;

  for (i = 0; i < _argc; i++) {
    /* Processing the next argument */
    str = _argv[i];
    printf("#argv[%d]=[%s]\n", i, str);
  }
}

int main(int argc, char *argv[])
{
  Database database;
  Eventset eventset;

	/*
	 * Body
	 */
	printf("Loading sequences...\n");
	database = db_read();

	printf("#database size:\t%d (seqs)\t%d (bytes)\n",
	       db_get_num_seqs(database), db_get_num_chars(database));
	db_print(database);

	printf("done.\n");
	exit(0);
}
